@@ -210,6 +210,7 @@ extern void pv_state_width_set(pvstate_t, unsigned int, bool);
 extern void pv_state_height_set(pvstate_t, unsigned int, bool);
 extern void pv_state_name_set(pvstate_t, /*@null@*/ const char *);
 extern void pv_state_format_string_set(pvstate_t, /*@null@*/ const char *);
+extern void pv_state_extra_display_set(pvstate_t, /*@null@*/ const char *);
 extern void pv_state_watch_pid_set(pvstate_t, pid_t);
 extern void pv_state_watch_fd_set(pvstate_t, int);
 extern void pv_state_output_set(pvstate_t, int, const char *);
@@ -237,7 +238,7 @@ extern off_t pv_calc_total_size(pvstate_t);
  */
 extern void pv_sig_init(pvstate_t);
 
-#ifdef SA_SIGINFO
+#ifdef SIGINFO_PROVIDES_PID
 /*
  * Return true if SIGUSR2 has been received, and indicate the sender.
  */
@@ -269,6 +270,10 @@ extern void pv_sig_fini(pvstate_t);
  */
 extern void pv_state_free(/*@only@*/ pvstate_t);
 
+#if ! HAVE_SETPROCTITLE
+void initproctitle(int, char **);
+void setproctitle(const char *, ...);
+#endif
 
 #ifdef ENABLE_DEBUGGING
 # if __STDC_VERSION__ < 199901L && !defined(__func__)
@@ -280,7 +285,7 @@ extern void pv_state_free(/*@only@*/ pvstate_t);
 # endif
 # define debug(x,...) debugging_output(__func__, __FILE__, __LINE__, x, __VA_ARGS__)
 #else
-# define debug(x,...) do { } while (0)
+# define debug(x,...) /*@-noeffect@*/ do { } while (0) /*@+noeffect@*/
 #endif
 
 /*
@@ -299,5 +304,3 @@ void debugging_output(const char *, const char *, int, const char *, ...);
 #endif
 
 #endif /* _PV_H */
-
-/* EOF */
