@@ -298,18 +298,9 @@ void display_help(void)
 		{ "-a", "--average-rate", NULL,
 		 N_("show data transfer average rate counter"),
 		 { 0, 0, 0, 0} },
-		{ "-m", "--average-rate-window", N_("SEC"),
-		 N_("compute average rate over past SEC seconds (default 30s)"),
-		 { 0, 0, 0, 0} },
 		{ "-b", "--bytes", NULL,
 		 N_("show number of bytes transferred"),
 		 { 0, 0, 0, 0} },
-		{ "-8", "--bits", NULL,
-		 N_("show number of bits transferred"),
-		 { 0, 0, 0, 0} },
-		{ "-k", "--si", NULL,
-		 N_("treat suffixes as multiples of 1000 rather than 1024"),
-		 { 0, 0, 0, 0} },
 		{ "-T", "--buffer-percent", NULL,
 		 N_("show percentage of transfer buffer in use"),
 		 { 0, 0, 0, 0} },
@@ -326,6 +317,12 @@ void display_help(void)
 		 N_("do not output any transfer information at all"),
 		 { 0, 0, 0, 0} },
 		{ "", NULL, NULL, NULL, { 0, 0, 0, 0} },
+		{ "-8", "--bits", NULL,
+		 N_("show number of bits transferred"),
+		 { 0, 0, 0, 0} },
+		{ "-k", "--si", NULL,
+		 N_("treat suffixes as multiples of 1000 rather than 1024"),
+		 { 0, 0, 0, 0} },
 		{ "-W", "--wait", NULL,
 		 N_("display nothing until first byte transferred"),
 		 { 0, 0, 0, 0} },
@@ -335,6 +332,9 @@ void display_help(void)
 		{ "-s", "--size", N_("SIZE"),
 		 N_("set estimated data size to SIZE bytes"),
 		 { 0, 0, 0, 0} },
+		{ "-g", "--gauge", NULL,
+		 N_("if size unknown, show rate vs max rate"),
+		 { 0, 0, 0, 0} },
 		{ "-l", "--line-mode", NULL,
 		 N_("count lines instead of bytes"),
 		 { 0, 0, 0, 0} },
@@ -344,6 +344,9 @@ void display_help(void)
 		{ "-i", "--interval", N_("SEC"),
 		 N_("update every SEC seconds"),
 		 { 0, 0, 0, 0} },
+		{ "-m", "--average-rate-window", N_("SEC"),
+		 N_("compute average rate over past SEC seconds (default 30s)"),
+		 { 0, 0, 0, 0} },
 		{ "-w", "--width", N_("WIDTH"),
 		 N_("assume terminal is WIDTH characters wide"),
 		 { 0, 0, 0, 0} },
@@ -353,6 +356,12 @@ void display_help(void)
 		{ "-N", "--name", N_("NAME"),
 		 N_("prefix visual information with NAME"),
 		 { 0, 0, 0, 0} },
+		{ "-x", "--extra-display", N_("SPEC"),
+		 N_("also send progress to SPEC"),
+		 { 0, 0, 0, 0} },
+		{ "-v", "--stats", NULL,
+		 N_("output transfer statistics at the end"),
+		 { 0, 0, 0, 0} },
 		{ "-f", "--force", NULL,
 		 N_("output even if standard error is not a terminal"),
 		 { 0, 0, 0, 0} },
@@ -390,20 +399,21 @@ void display_help(void)
 		{ "-X", "--discard", NULL,
 		 N_("discard input instead of writing to output"),
 		 { 0, 0, 0, 0} },
-#ifdef SA_SIGINFO
+		{ "-U", "--store-and-forward", N_("FILE"),
+		 N_("write all input to FILE before writing to output"),
+		 { 0, 0, 0, 0} },
+		{ "-d", "--watchfd", N_("PID[:FD]"),
+		 N_("watch file FD opened by process PID"),
+		 { 0, 0, 0, 0} },
+#ifdef SIGINFO_PROVIDES_PID
 		{ "-R", "--remote", N_("PID"),
 		 N_("update settings of process PID"),
 		 { 0, 0, 0, 0} },
-#endif				/* SA_SIGINFO */
+#endif				/* SIGINFO_PROVIDES_PID */
 		{ "", NULL, NULL, NULL, { 0, 0, 0, 0} },
 		{ "-P", "--pidfile", N_("FILE"),
 		 N_("save process ID in FILE"),
 		 { 0, 0, 0, 0} },
-		{ "", NULL, NULL, NULL, { 0, 0, 0, 0} },
-		{ "-d", "--watchfd", N_("PID[:FD]"),
-		 N_("watch file FD opened by process PID"),
-		 { 0, 0, 0, 0} },
-		{ "", NULL, NULL, NULL, { 0, 0, 0, 0} },
 		{ "-h", "--help", NULL,
 		 N_("show this help and exit"),
 		 { 0, 0, 0, 0} },
@@ -598,5 +608,3 @@ void display_help(void)
 
 	printf("\n");
 }
-
-/* EOF */
