@@ -1,7 +1,7 @@
 /*
  * Remote-control functions.
  *
- * Copyright 2002-2008, 2010, 2012-2015, 2017, 2021, 2023 Andrew Wood
+ * Copyright 2002-2008, 2010, 2012-2015, 2017, 2021, 2023-2024 Andrew Wood
  *
  * License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
  */
@@ -21,7 +21,7 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 
-#ifdef SA_SIGINFO
+#ifdef SIGINFO_PROVIDES_PID
 void pv_error(pvstate_t, char *, ...);
 
 struct remote_msg {
@@ -57,10 +57,12 @@ struct remote_msg {
  * file associated with a particular process ID; it will be opened for
  * writing if "sender" is true.  Returns NULL on error.
  */
+/*@null@ */
+/*@dependent@ */
 static FILE *pv__control_file(char *filename, size_t bufsize, pid_t control_pid, bool sender)
 {
 	int open_flags, open_mode, control_fd;
-	FILE *control_fptr;
+	/*@dependent@ */ FILE *control_fptr = NULL;
 
 	open_flags = O_RDONLY;
 #ifdef O_NOFOLLOW
@@ -84,6 +86,13 @@ static FILE *pv__control_file(char *filename, size_t bufsize, pid_t control_pid,
 		home_dir = getenv("HOME");  /* flawfinder: ignore */
 		if (NULL == home_dir)
 			return NULL;
+		if ('\0' == home_dir[0])
+			return NULL;
+
+		/*
+		 * flawfinder rationale: null and zero-size values are
+		 * rejected, and the destination buffer is bounded.
+		 */
 
 		(void) pv_snprintf(filename, bufsize, "%s/.pv/remote.%lu", home_dir, (unsigned long) control_pid);
 		control_fd = open(filename, open_flags, open_mode);	/* flawfinder: ignore */
@@ -151,7 +160,7 @@ int pv_remote_set(opts_t opts, pvstate_t state)
 	 */
 	if (kill((pid_t) (opts->remote), 0) != 0) {
 		pv_error(state, "%u: %s", opts->remote, strerror(errno));
-		return 1;
+		return PV_ERROREXIT_REMOTE_OR_PID;
 	}
 
 	/*
@@ -209,10 +218,10 @@ int pv_remote_set(opts_t opts, pvstate_t state)
 	 * Get the filename and file stream to use for remote control.
 	 */
 	memset(control_filename, 0, sizeof(control_filename));
-	control_fptr = pv__control_file(control_filename, sizeof(control_filename), getpid(), true);
+	control_fptr = pv__control_file(control_filename, sizeof(control_filename), (pid_t) getpid(), true);
 	if (NULL == control_fptr) {
 		pv_error(state, "%s", strerror(errno));
-		return 1;
+		return PV_ERROREXIT_REMOTE_OR_PID;
 	}
 
 	/*
@@ -223,13 +232,13 @@ int pv_remote_set(opts_t opts, pvstate_t state)
 		pv_error(state, "%s", strerror(errno));
 		(void) fclose(control_fptr);
 		(void) remove(control_filename);
-		return 1;
+		return PV_ERROREXIT_REMOTE_OR_PID;
 	}
 
 	if (0 != fclose(control_fptr)) {
 		pv_error(state, "%s", strerror(errno));
 		(void) remove(control_filename);
-		return 1;
+		return PV_ERROREXIT_REMOTE_OR_PID;
 	}
 
 	/*
@@ -241,7 +250,7 @@ int pv_remote_set(opts_t opts, pvstate_t state)
 	if (kill((pid_t) (opts->remote), SIGUSR2) != 0) {
 		pv_error(state, "%u: %s", opts->remote, strerror(errno));
 		(void) remove(control_filename);
-		return 1;
+		return PV_ERROREXIT_REMOTE_OR_PID;
 	}
 
 	debug("%s", "message sent");
@@ -293,7 +302,7 @@ int pv_remote_set(opts_t opts, pvstate_t state)
 	 * fact we only translate each string once.
 	 */
 	pv_error(state, "%u: %s", opts->remote, _("message not received"));
-	return 1;
+	return PV_ERROREXIT_REMOTE_OR_PID;
 	/*@+mustfreefresh @ */
 }
 
@@ -324,7 +333,7 @@ void pv_remote_check(pvstate_t state)
 	memset(control_filename, 0, sizeof(control_filename));
 	control_fptr = pv__control_file(control_filename, sizeof(control_filename), signal_sender, false);
 	if (NULL == control_fptr) {
-		pv_error(state, "%s", strerror(errno));
+		pv_error(state, "%s: %s", control_filename, strerror(errno));
 		return;
 	}
 
@@ -398,10 +407,10 @@ void pv_remote_fini(void)
 {
 }
 
-#else				/* !SA_SIGINFO */
+#else				/* !SIGINFO_PROVIDES_PID */
 
 /*
- * Dummy stubs for remote control when we don't have SA_SIGINFO.
+ * Dummy stubs for remote control when we don't have SIGINFO_PROVIDES_PID.
  */
 void pv_remote_init(void)
 {
@@ -421,9 +430,7 @@ int pv_remote_set(			 /*@unused@ */
 	/*@-mustfreefresh@ *//* splint - see above */
 	fprintf(stderr, "%s\n", _("SA_SIGINFO not supported on this system"));
 	/*@+mustfreefresh@ */
-	return 1;
+	return PV_ERROREXIT_REMOTE_OR_PID;
 }
 
 #endif				/* SA_SIGINFO */
-
-/* EOF */
