@@ -43,32 +43,6 @@
 #endif
 
 
-/*
- * Write the given buffer to the given file descriptor, retrying until all
- * bytes have been written or an error has occurred.
- */
-void pv_write_retry(int fd, const char *buf, size_t count)
-{
-	while (count > 0) {
-		ssize_t nwritten;
-
-		nwritten = write(fd, buf, count);
-
-		if (nwritten < 0) {
-			if ((EINTR == errno) || (EAGAIN == errno)) {
-				continue;
-			}
-			return;
-		}
-		if (nwritten < 1)
-			return;
-
-		count -= nwritten;
-		buf += nwritten;
-	}
-}
-
-
 /*
  * Create a per-euid, per-tty, lockfile in ${TMPDIR:-${TMP:-/tmp}} for the
  * tty on the given file descriptor.
@@ -437,8 +411,8 @@ void pv_crs_init(pvstate_t state)
 	 * If we have already set the terminal TOSTOP attribute, set the
 	 * flag in shared memory to let the other instances know.
 	 */
-	if ((!state->cursor.noipc) && state->signal.pv_tty_tostop_added && (NULL != state->cursor.shared)) {
-		debug("%s", "propagating local pv_tty_tostop_added true value to shared flag");
+	if ((!state->cursor.noipc) && (1 == state->flag.clear_tty_tostop_on_exit) && (NULL != state->cursor.shared)) {
+		debug("%s", "propagating local clear_tty_tostop_on_exit true value to shared tty_tostop_added flag");
 		state->cursor.shared->tty_tostop_added = true;
 	}
 
@@ -462,7 +436,7 @@ void pv_crs_init(pvstate_t state)
 		 * initial ypos.
 		 */
 		if (state->cursor.y_start > 0)
-			pv_write_retry(STDERR_FILENO, "\n", 1);
+			pv_tty_write(state, "\n", 1);
 		pv_crs_unlock(state, terminalfd);
 
 		if (state->cursor.y_start < 1)
@@ -495,6 +469,11 @@ static void pv_crs_reinit(pvstate_t state)
 {
 	debug("%s", "reinit");
 
+	if (1 == state->flag.suspend_stderr) {
+		debug("%s", "reinit abandoned - stderr is suspended");
+		return;
+	}
+
 	pv_crs_lock(state, STDERR_FILENO);
 
 	state->cursor.needreinit--;
@@ -589,9 +568,9 @@ void pv_crs_update(pvstate_t state, const char *output_line)
 			memset(cup_cmd, 0, sizeof(cup_cmd));
 			(void) pv_snprintf(cup_cmd, sizeof(cup_cmd), "\033[%u;1H", state->control.height);
 			cup_cmd_length = strlen(cup_cmd);	/* flawfinder: ignore */
-			pv_write_retry(STDERR_FILENO, cup_cmd, cup_cmd_length);
+			pv_tty_write(state, cup_cmd, cup_cmd_length);
 			for (; offs > 0; offs--) {
-				pv_write_retry(STDERR_FILENO, "\n", 1);
+				pv_tty_write(state, "\n", 1);
 			}
 
 			pv_crs_unlock(state, STDERR_FILENO);
@@ -624,8 +603,8 @@ void pv_crs_update(pvstate_t state, const char *output_line)
 
 	pv_crs_lock(state, STDERR_FILENO);
 
-	pv_write_retry(STDERR_FILENO, cup_cmd, cup_cmd_length);
-	pv_write_retry(STDERR_FILENO, output_line, output_line_length);
+	pv_tty_write(state, cup_cmd, cup_cmd_length);
+	pv_tty_write(state, output_line, output_line_length);
 
 	pv_crs_unlock(state, STDERR_FILENO);
 }
@@ -664,7 +643,7 @@ void pv_crs_fini(pvstate_t state)
 
 	pv_crs_lock(state, STDERR_FILENO);
 
-	pv_write_retry(STDERR_FILENO, cup_cmd, strlen(cup_cmd));	/* flawfinder: ignore */
+	pv_tty_write(state, cup_cmd, strlen(cup_cmd));	/* flawfinder: ignore */
 	/* flawfinder - pv_snprintf() always \0-terminates (see above). */
 
 #ifdef HAVE_IPC
@@ -674,9 +653,10 @@ void pv_crs_fini(pvstate_t state)
 	 * it.
 	 */
 	if ((!state->cursor.noipc) && (NULL != state->cursor.shared) && state->cursor.shared->tty_tostop_added) {
-		if (!state->signal.pv_tty_tostop_added) {
-			debug("%s", "propagating shared tty_tostop_added true value to local flag");
-			state->signal.pv_tty_tostop_added = true;
+		if (0 == state->flag.clear_tty_tostop_on_exit) {
+			debug("%s",
+			      "propagating shared tty_tostop_added true value to local clear_tty_tostop_on_exit flag");
+			state->flag.clear_tty_tostop_on_exit = 1;
 		}
 	}
 
@@ -709,5 +689,3 @@ void pv_crs_fini(pvstate_t state)
 		(void) remove(state->cursor.lock_file);
 	}
 }
-
-/* EOF */
