@@ -636,6 +636,7 @@ static int pv__transfer_read(pvstate_t state, int fd, bool *eof_in, bool *eof_ou
 static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long *lineswritten)
 {
 	ssize_t nwritten;
+	int write_errno;
 
 	if (NULL == state->transfer.transfer_buffer) {
 		pv_error(state, "%s", _("no transfer buffer allocated"));
@@ -646,6 +647,7 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 	}
 
 	nwritten = 0;
+	write_errno = 0;
 
 	if (state->control.discard_input) {
 		nwritten = state->transfer.to_write;
@@ -693,6 +695,7 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 						       (size_t) (state->transfer.to_write),
 						       state->control.sync_after_write);
 		if (nwritten < 0) {
+			write_errno = (int) errno;
 			debug("%s: %ld: %s", "bytes written", (long) nwritten, strerror(errno));
 		} else {
 			debug("%s: %ld", "bytes written", (long) nwritten);
@@ -723,6 +726,22 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 			char *ptr;
 			long lines = 0;
 
+			/*
+			 * Line mode - look through what we've just written
+			 * to count how many lines there were.
+			 */
+
+			/* Allocate buffer to remember line positions. */
+			if (NULL == state->transfer.line_positions) {
+				state->transfer.line_positions_capacity = MAX_LINE_POSITIONS;
+				state->transfer.line_positions =
+				    calloc((size_t) (state->transfer.line_positions_capacity), sizeof(off_t));
+				if (NULL == state->transfer.line_positions) {
+					pv_error(state, "%s: %s", _("line position buffer allocation failed"),
+						 strerror(errno));
+				}
+			}
+
 			if (state->control.null_terminated_lines) {
 				separator = '\0';
 			} else {
@@ -732,9 +751,30 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 			ptr = (char *) (state->transfer.transfer_buffer + state->transfer.write_position - 1);
 			for (ptr++;
 			     ptr - (char *) state->transfer.transfer_buffer - state->transfer.write_position <
-			     (size_t) nwritten; ptr++) {
-				if (*ptr == separator)
-					++lines;
+			     (size_t) nwritten; ptr++, state->transfer.last_output_position++) {
+				if (*ptr != separator)
+					continue;
+
+				/* Separator found - increment line count. */
+				++lines;
+
+				if (NULL == state->transfer.line_positions)
+					continue;
+
+				/* Store the position of the separator. */
+				state->transfer.line_positions[state->transfer.line_positions_head] =
+				    state->transfer.last_output_position;
+				state->transfer.line_positions_head++;
+
+				/* Circular buffer - wrap around. */
+				if (state->transfer.line_positions_head >= state->transfer.line_positions_capacity) {
+					state->transfer.line_positions_head = 0;
+				}
+
+				/* Increment count of line positions, if below capacity. */
+				if (state->transfer.line_positions_length < state->transfer.line_positions_capacity) {
+					state->transfer.line_positions_length++;
+				}
 			}
 
 			*lineswritten += lines;
@@ -808,11 +848,11 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 	 * blocked on first write such that nwritten == 0, just wait a bit and
 	 * then return zero, since this was a transient error.
 	 */
-	if ((0 == nwritten) || (EINTR == errno) || (EAGAIN == errno)) {
+	if ((0 == nwritten) || (EINTR == write_errno) || (EAGAIN == write_errno)) {
 		if (0 == nwritten) {
 			debug("%s", "attempted write blocked - waiting briefly");
 		} else {
-			debug("%s: %s", "transient write error - waiting briefly", strerror(errno));
+			debug("%s: %s", "transient write error - waiting briefly", strerror(write_errno));
 		}
 		(void) is_data_ready(-1, NULL, -1, NULL, 10000);
 		return 0;
@@ -822,13 +862,14 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 	 * SIGPIPE means we've finished. Don't output an error because it's
 	 * not really our error to report.
 	 */
-	if (EPIPE == errno) {
+	if (EPIPE == write_errno) {
 		*eof_in = true;
 		*eof_out = true;
+		state->flag.pipe_closed = 1;
 		return 0;
 	}
 
-	pv_error(state, "%s: %s", _("write failed"), strerror(errno));
+	pv_error(state, "%s: %s", _("write failed"), strerror(write_errno));
 	state->status.exit_status |= PV_ERROREXIT_TRANSFER;
 	*eof_out = true;
 	state->transfer.written = -1;
@@ -1141,5 +1182,3 @@ ssize_t pv_transfer(pvstate_t state, int fd, bool *eof_in, bool *eof_out, off_t
 
 	return state->transfer.written;
 }
-
-/* EOF */
